*===============================================================================
$ontext
FILENAME:        link.gms
DATE:            3 July, 2002

PURPOSE:         Creates a .tab MAPINFO network-type file with links.
                 The links type can be between "states" (for state centroids), 
	         "world" (for world country centroids), "uscaps" (for US state
	  	 capitals), or "worldcaps" (for world country capitals).
                 Links cannot be between different point types
                 (e.g. we cannot link a state and a county). If a data with
                 numeric values associated with points is specified, these
                 values are represented by a width (See the user options below).

DATA INPUT:      Data is of the form:
                         pt1     pt2     num.value
                 For example:
                         TX      SD         5.25
                 If not numeric value is to be associated with a link, then
                 all numerical values should be set to 1.

REMARK:          
                 If bidirectional links are specified (e.g. TX --> SD and
                 SD --> TX), then we get TX <--> SD with the color and width
                 those of the link with the larger absolute value |num.value|.
                 This will be shown in the status.log file.

USER OPTIONS:
  --color=                       Default is black. Applies only if all numerical
                                   values are set to 1 (i.e. no numerical data).
                                   The option --color specifies the color of the
                                   links between points specified in the
                                   --datafile.

  --colp=                        Default is green. Applies only if a --datafile
                                   with numerical values is included. Color of
                                   those links having positive numerical value

  --coln=                        Default is red. Applies only if a --datafile
                                   with numerical values is included. Color of
                                   those links having negative numerical value

  --data=                        Default is 'states'. Point type. Can be either
                                   'states', 'world', 'uscaps', or 'worldcaps.'

  --datafile=                    The default is the sample file 'links.txt'.
                                   A --datafile must be included. The --datafile
                                   contains the starting and ending point ID of
                                   the link and possibly a numerical value
                                   associated with that link. The --datafile
                                   is a .txt file. Input example:
                                         point_ID1  point_ID2   (99.97)
                                   If no numerical values are desired, then set
                                   all values to 1.

  --linkstyle                    Default is 0 (which is arrow). Mapinfo line style.
                                 2 = line, 3=dotted line. See the MapInfo Users
                                 Manual on line styles. 

  --noplot=                      Default is none. If a subset of links from the
                                   --datafile is not to be plotted. Format is
                                   pt1.pt2 (seperated by a .dot). For example,
                                   --noplot="VA.DC, TX.SD"

  --plot=                        Default is all in --datafile. If only a subset
                                   of links from the --datafile is to be plotted.
                                   Format is pt1.pt2 (seperated by a .dot). For
                                   example, --noplot="VA.DC, TX.SD"

  --scale                        Default is scaling.gms. GAMS scaling file for
                                   custom scaling of region shading, point sizes,
                                   and link widths.

  --scaleopt                     Default=0. 0=[0,max] 1=[min,max] 

  --tabfn=(output filename)      Default is "links.tab"

  --width=                       Default is 1. Width of the link. The --width
                                   option applies only if no numerical values
                                   are given in the --datafile.

  --xoffset                      Default is 0. Offset in the x direction from
                                   actual coordinate data.

  --yoffset                      Default is 0. Offset in the y direction from
                                   actual coordinate data.

SAMPLE COMMANDS:

1. For state link map (with numerical values):
  gams link --tabfn=states_link --datafile=statedata.txt  --colp=red --coln=black

$offtext
*===============================================================================
$eolcom //

$if not set database  $set database GMSMAPDB.zip 

* Copy GMSMAPDB database to current directory
$onecho > tmp
   file fcopy / copydatabase.gms /;
   put fcopy '$call copy %db% TMPGMSMAPDB.zip'; 
$offecho
$call gams tmp --db=%database%
$call gams copydatabase.gms
$call rm tmp.* copydatabase.*


$call gmsunzip -Cp TMPGMSMAPDB initialize.gms > "%gams.scrdir%initialize.scr"
$include "%gams.scrdir%initialize.scr"


* Define user options
$if not set color        $set color black
$if not set colp         $set colp green
$if not set coln         $set coln red
$if not set data         $set data states
$if not set datafile     $set datafile links.txt
$if not set dborig       $set dborig orig
$if not set dbdest       $set dbdest dest
$if not set linkstyle    $set linkstyle 0
$if not set plot         $set plot
$if not set noplot       $set noplot
$if not set scaleopt     $set scaleopt 0
$if not set tabfn        $set tabfn links
$if not set width        $set width 0
$if not set xoffset      $set xoffset 0
$if not set yoffset      $set yoffset 0

$if not set scale        $goto nocscal

* User defined scaling options for link width (based on numerical values).
$kill steps_w scaling_w
$kill steps_s scaling_s
$kill steps   scaling
$include %scale%

$label nocscal

$onempty
Set link_plot(*,*)  links to be plotted /
%plot%
/;
$offempty
* If no links are specified upon input for --plot, we assume we plot all links


Alias(data2,%data%);
if (card(link_plot)=0,
  link_plot(%data%,data2) = YES;
)


$onempty
Set link_noplot(*,*)  links to be ignored during plotting /
%noplot%
/;
$offempty



* Call custom dBase script if --dbasescript is specified
$if     set dbasescript $call dbase %dbasescript%

* Skip dbase if no --dbfield is specified
$if not set dbfield $goto nodbase

* Begin extracting data from dbase file (uses default file dbaselink.gms)
$if not set dbasescript $call gmsunzip -Cp TMPGMSMAPDB dbaselink.gms > "%gams.scrdir%dbaselink.scr"
$if not set dbasescript $include "%gams.scrdir%dbaselink.scr"
$if not set dbasescript $set dbasescript x.prg


$call  dbase %dbasescript%


$label nodbase


* Set of numerical data for region (if specified)
$if     set dbfield $set ifn %datafile%.txt
$if not set dbfield $set ifn %datafile%


* Set of numerical data for region (if specified)
$onempty
Table link_num(*,*,*)
           value
$ondelim
$if exist %ifn% $include %ifn%
$offdelim
;
$offempty



* Write data to MIF/MID files
$call gmsunzip -Cp TMPGMSMAPDB write_link.gms > "%gams.scrdir%write_link.scr"
$  batinclude "%gams.scrdir%write_link.scr"  %data%  %data%coord link_num  link_plot  link_noplot  %tabfn%  %width% %color% %colp% %coln%  %xoffset%  %yoffset%  %scaleopt%  %linkstyle%

* Convert to .tab file
execute 'tab2tab %tabfn%.mif %tabfn%.tab';

* Output status file
execute 'type status.log';


* Clean up
$call rm TMPGMSMAPDB.zip