
*===============================================================================
$ontext
FILENAME:        point.gms
DATE:            3 July, 2002

PURPOSE:         Creates a .tab MAPINFO file with points. The point type can
                 be either "states" (for state centroids), "world" (for
	         world country centroids), uscaps (for US state capitals),
	         or worldcaps (for world country capitals). The user can
                 input a point data file to associate numeric data with each
	         point.


USER OPTIONS:
  --color                        Default is black. If no datafile is included,
                                   then --color specifies the point interior
                                   color. If a --datafile is included, then
                                   --color specifies the point interior color
                                   of those points not specified in the
                                   datafile.

  --colp                         Default is green. Applies only if a --datafile
                                   is specified and --type=color. Shading of
                                   positive numeric values associated with each
                                   point in the datafile.

  --coln                         Default is red. Applies only if a --datafile is
                                   specified and --type=color. Shading of negative
                                   numeric values associated with each point in
                                   the --datafile.

  --data                         Default is 'states'. Point type. Can be either
                                   'states', 'world', 'uscaps', or 'worldcaps.'

  --database=                    Name and path of GAMSMAP database (default is 
                                   GMSMAPDB.zip in current directory)

  --datafile                     Default is none. Numeric data input file.

  --noplot                       Default is "AK,HI, Canadian states". Plants/regions
                                   specified in --noplot are ignored and not
                                   shown in map. If a point is specified in both
                                   --plot and --noplot, then --noplot overrides.
                                   If a point specified in --noplot does not
                                   exist in the total set of that type (for example
                                   TX is not a control area), then it is ignored.

  --plot='VA, DC, CA'            Default is all points.
                                   If the --plot option is included only those
                                   points specified in --plot are included.
                                   If a point in --plot is specified for which
                                   no data exists, the region is ignored, but
                                   a warning message appears in status.log file.
                                   If a point is specified in both --plot and
                                   --noplot, then --noplot overrides.
                                   If a point specified in --plot does not
                                   exist in the total set of that type (for example
                                   TX is not a control area), then it is ignored.

  --scale                        Default is scaling.gms. GAMS scaling file for
                                   custom scaling of region shading, point sizes,
                                   and link widths.

  --scaleopt                     Default=0. 0=[0,max] 1=[min,max] 

  --size                         Default is 0. Applies only if no --datafile is
                                   included or if a --datafile is included, the
                                   --type=color.

  --tabfn=(output filename)      Default is "points.tab"

  --type                         Default is "size". Applies only if a --datafile
                                   is included. User may choose from --type=size
                                   (numerical values are shown by size) or
                                   --type=color (numerical values are shown by
                                   shading).

  --xoffset                      Default is 0. Offset in the x direction from
                                   actual coordinate data.

  --yoffset                      Default is 0. Offset in the y direction from
                                   actual coordinate data.


SAMPLE COMMANDS:

1. For world country centroids coordinate map:
  gams points  --tabfn=world --color=blue --size=15

2. For state map with text data file:
  gams plants  --tabfn=states --data=states --datafile=newc_10 --color=none 
               --colp=yellow --coln=red  --type=size

$offtext
*===============================================================================
$eolcom //


$if not set database  $set database GMSMAPDB.zip 

* Copy GMSMAPDB database to current directory
$onecho > tmp
   file fcopy / copydatabase.gms /;
   put fcopy '$call copy %db% TMPGMSMAPDB.zip'; 
$offecho
$call gams tmp --db=%database%
$call gams copydatabase.gms
$call rm tmp.* copydatabase.*


$call gmsunzip -Cp TMPGMSMAPDB initialize.gms > "%gams.scrdir%initialize.scr"
$include "%gams.scrdir%initialize.scr"

* Define user options
$if not set color    $set color black
$if not set colp     $set colp green
$if not set coln     $set coln red
$if not set data     $set data states
$if not set datafile $set datafile
$if not set dbindex  $set dbindex
$if not set noplot   $set noplot
$if not set plot     $set plot
$                     set shade solid
$if not set scaleopt $set scaleopt 0
$if not set size     $set size 0
$if not set tabfn    $set tabfn points
$if not set type     $set type size
$if not set xoffset  $set xoffset 0
$if not set yoffset  $set yoffset 0

$if not set scale        $goto nocscal

* User defined scaling options for region color shading (based on numerical values).
$kill steps_w scaling_w
$kill steps_s scaling_s
$kill steps   scaling
$include %scale%

$label nocscal


$onempty
Set pt_plot(*)  points to be plotted /
%plot%
/;
$offempty

* If no points are specified upon input for --plot, we assume we plot all points
pt_plot(%data%)$(card(pt_plot)=0) = yes;

$onempty
Set pt_noplot(*)   points to be ignored during plotting /
%noplot%
/;
$offempty

set def_noplot(*) / empty, AK, HI /;

if (card(pt_noplot)=0,
   pt_noplot(def_noplot) = yes;
   pt_noplot(ca)         = yes;
else if (card(pt_noplot)=1 and pt_noplot('empty'),
   option clear=pt_noplot;
  )
);


* Call custom dBase script if --dbasescript is specified
$if     set dbasescript $call dbase %dbasescript%
$if     set dbasescript $goto nodbase


* Skip dbase if no --dbfield is specified
$if not set dbfield $goto nodbase

* Begin extracting data from dbase file (uses default file dbase.gms)
$if not set dbasescript $call gmsunzip -Cp TMPGMSMAPDB dbase.gms > "%gams.scrdir%dbase.scr"
$if not set dbasescript $include "%gams.scrdir%dbase.scr"
$if not set dbasescript $set dbasescript x.prg
$call  dbase %dbasescript%

$label nodbase


* Set of numerical data for region (if specified)
$if     set dbfield $set ifn %datafile%.txt
$if not set dbfield $set ifn %datafile%


$onempty
Parameter pt_num(*) /
$ondelim
$if  exist %ifn% $include %ifn%
$offdelim
/;

* Write data to MIF/MID files
$call gmsunzip -Cp TMPGMSMAPDB write_point.gms > "%gams.scrdir%write_point.scr"
$batinclude "%gams.scrdir%write_point.scr"  %data%  %data%coord pt_num  pt_plot  pt_noplot  %tabfn%  %size% %color% %colp% %coln%  %type%  %xoffset%  %yoffset%     %scaleopt%

* Convert to .tab file
execute 'tab2tab %tabfn%.mif %tabfn%.tab';

* Output status file
execute 'type status.log';


* Clean up
$call rm TMPGMSMAPDB.zip