*===============================================================================
$ontext
FILENAME:        region.gms
DATE:            3 July, 2002

PURPOSE:         Creates a .tab MAPINFO file with regions. The region type can
                 be either "states", or "world".
 
USER OPTIONS:
  --color=                       Default is lightyellow. If no datafile is included,
                                   then --color specifies the region interior
                                   color. If a dbfile is included, then
                                   --color specifies the region interior color
                                   of those regions not specified in the
                                   datafile.

  --colorfile=                   Default is none. If regions are to be colored
                                   according to file.

  --colp=                        Default is green. Applies only if a --datafile
                                   is specified. Shading of positive numeric
                                   values associated with each region in the
                                   datafile.

  --coln=                        Default is red. Applies only if a --datafile is
                                   specified. Shading of negative numeric values
                                   associated with each region in the --datafile.

  --data=                        Default is 'states'. Region type. Can be either
                                   'states', or 'world'.

  --database=                    Name and path of GAMSMAP database (default is 
                                   GMSMAPDB.zip in current directory)

  --datafile=                    Data textfile. Default is none.

  --noplot=                      Default is "AK,HI, Canadian states". Regions
                                   specified in --noplot are ignored and not
                                   shown in map. If a region is specified in both
                                   --plot and --noplot, then --noplot overrides.
                                   If a region specified in --noplot does not
                                   exist in the total set of that type (for example
                                   TX is not a control area), then it is ignored.

  --plot='VA, DC, CA'            Default is all regions.
                                   If the --plot option is included only those
                                   regions specified in --plot are included.
                                   If a region in --plot is specified for which
                                   no data exists, the region is ignored, but
                                   a warning message appears in status.log file.
                                   If a region is specified in both --plot and
                                   --noplot, then --noplot overrides.
                                   If a region specified in --plot does not
                                   exist in the total set of that type (for example
                                   TX is not a control area), then it is ignored.

  --scale                        Default is scaling.gms. GAMS scaling file for
                                   custom scaling of region shading, point sizes,
                                   and link widths.

  --scaleopt                     Default=0. 0=[0,max] 1=[min,max] 

  --shade=                       Default is solid if no --color is specified and
                                   solid is a --color is specified. If a data
                                   file is included, those regions listed in the
                                   data file have a solid shading.
                                   The user may choose from  solid 2, fade 50,
                                   lightfade 49, none 1.

  --tabfn=(output filename)      Default is "region.tab"

  --width                        Default is 1. Border width.

SAMPLE COMMANDS:

For state map with data file:

  gams region --tabfn=states --data=states --datafile=statedata.txt
                  --color=none --colp=yellow --coln=red --width=3

$offtext
*===============================================================================
$eolcom //

$if not set database  $set database GMSMAPDB.zip 

* Copy GMSMAPDB database to current directory
$onecho > tmp
   file fcopy / copydatabase.gms /;
   put fcopy '$call copy %db% TMPGMSMAPDB.zip'; 
$offecho
$call gams tmp --db=%database%
$call gams copydatabase.gms
$call rm tmp.* copydatabase.*



$call gmsunzip -Cp TMPGMSMAPDB initialize.gms > "%gams.scrdir%initialize.scr"
$include "%gams.scrdir%initialize.scr"


* Define user options
$if not set color        $set color lightyellow
$if not set colp         $set colp green
$if not set coln         $set coln red
$if not set data         $set data states
$if not set dbindex      $set dbindex state
$if not set noplot       $set noplot
$if not set plot         $set plot
$if not set scaleopt     $set scaleopt 0
$if not set shade        $set shade solid
$if not set tabfn        $set tabfn region
$if not set width        $set width 1

$if not set colorfile    $goto nocolorfile  

* If --colorfile defined set region colors to file colors
Parameter reg_color_tmp(*)/
$onempty
$include %colorfile%
$offempty
/;
$goto colorfiledone

* No --colorfile defined
$label nocolorfile
Parameter reg_color_tmp(*);
reg_color_tmp(%data%) = 0;
$if not set colorfile $set colorfile none
$label colorfiledone

$if not set scale        $goto nocscal

$kill steps_w scaling_w
$kill steps_s scaling_s
$kill steps   scaling
$include %scale%

$label nocscal

$onempty
Set reg_plot(*)  regions to be plotted /
%plot%
/;
$offempty


* If no regions are specified upon input for --plot, we assume we plot all regions
reg_plot(%data%)$(card(reg_plot)=0) = 1;


$onempty
Set reg_noplot(*)  regions to be ignored during plotting /
%noplot%
/;
$offempty

set def_noplot(*) / empty, AK, HI /;

if (card(reg_noplot)=0,
   reg_noplot(def_noplot) = yes;
   reg_noplot(ca)         = yes;
else if (card(reg_noplot)=1 and reg_noplot('empty'),
   option clear=reg_noplot;
  )
);


* Call custom dBase script if --dbasescript is specified
$if     set dbasescript $call dbase %dbasescript%
$if     set dbasescript $goto nodbase 

* Skip dbase if no --dbfield is specified
$if not set dbfield $goto nodbase

* Begin extracting data from dbase file (uses default file dbase.gms)
$if not set dbasescript $call gmsunzip -Cp TMPGMSMAPDB dbase.gms > "%gams.scrdir%dbase.scr"
$if not set dbasescript $include "%gams.scrdir%dbase.scr"
$if not set dbasescript $set dbasescript x.prg
$call  dbase %dbasescript%

$label nodbase

* Set of numerical data for region (if specified)
$if     set dbfield $set ifn "%datafile%.txt"
$if not set dbfield $set ifn "%datafile%"


$onempty
Parameter reg_num(*) /
$ondelim
$if exist %ifn% $include %ifn%
$offdelim
/;



* Write data to MIF/MID files
$call gmsunzip -Cp TMPGMSMAPDB write_region.gms > "%gams.scrdir%write_region.scr"
$batinclude "%gams.scrdir%write_region.scr"  %data%  reg_num  reg_plot  reg_noplot  %tabfn%  %width% %color% shading("%shade%") %colp% %coln%   %scaleopt%  %colorfile%  reg_color_tmp

* Convert to .tab file
execute 'tab2tab %tabfn%.mif %tabfn%.tab';

* Output status file
execute 'type status.log';


* Clean up
$call rm TMPGMSMAPDB.zip