*===============================================================================
$ontext
FILENAME:      write_link.gms
DATE:          7 May, 2002
PURPOSE:       Creates the MIF/MID files depicting links between points
                         (plants, centroids of states, centroids of control
                         areas, etc.)
INPUT PARAMETERS:
               pt          -  set of points (either plants, centroids of states,
                                 control areas, counties, etc.)
               pt_coord    -  set of coordinate data for point type pt.
               link_num    -  set of links with numerical values in data file
                                 for which mapping data exists
               link_plot   -  set of links to actually be plotted
               link_noplot -  set of links to be ignored
               fname       -  .mif and .mid root filename
               pt_width    -  size of point
               color       -  color of point interior
               colp        -  shading color of positive numerical values in pt_num
               coln        -  shading color of negative numerical values in pt_num
               pt_type     -  if a data file with numerical values is specified,
                                 the points can be displayed by size or by shading
               link_xoffset-  if specified, x-direction offset from actual coordinate data
               link_yoffset-  if specified, y-direction offset from actual coordinate data
               scaleopt    -  scaling option. 1=[0,max], 2=[min,max]
               linkstyle   -  style of link (for example 59=arrow, 2=line); 

CALLED BY:
*$  batinclude write_link.gms  pt  pt_coord  link_num  link_plot  ...
                         ... link_noplot  fname  width  color  colp  coln
                         ... link_xoffset  link_yoffset  scaleopt linkstyle


$offtext

*===============================================================================

$call gmsunzip -Cp GMSMAPDB get_width.gms > "%gams.scrdir%get_width.scr"

* Create status report
put flog ''; // deletes flog
parameter runi /0/;


* Input identifiers
$set pt           %1
$set pt_coord     %2
$set link_num     %3
$set link_plot    %4
$set link_noplot  %5
$set fname        %6
$set width        %7
$set color        %8
$set link_colp    %9
$set link_coln    %10
$set link_xoffset %11
$set link_yoffset %12
$set scaleopt     %13
$set linkstyle    %14


* Define link types
$set line        02
$set arrow       59
$set rev_arrow   60
$set dbl_arrow   61


Alias (pt2,%pt%);

* Identifier if numerical values are to be associated with each pt
num_ident$(card(%link_num%)>0) = 1;

* Compute max and min ranges for numerical value %pt_num%(pt)
max_p = smax((%pt%,pt2)$(%link_num%(%pt%,pt2,'value')>0), %link_num%(%pt%,pt2,'value'));
max_n = smax((%pt%,pt2)$(%link_num%(%pt%,pt2,'value')<0), abs(%link_num%(%pt%,pt2,'value')) );

min_p = smin((%pt%,pt2)$(%link_num%(%pt%,pt2,'value')>0), %link_num%(%pt%,pt2,'value'));
min_n = smin((%pt%,pt2)$(%link_num%(%pt%,pt2,'value')<0), abs(%link_num%(%pt%,pt2,'value')) );

maxval = max(max_p, max_n);
minval = min(min_p, min_n);


link_color(%pt%,pt2) = irgb("%color%");
*link_width(%pt%,pt2) = %width%;


parameter linknumber;
* Define color and width values for regions in data file
loop((%pt%,pt2)$( %link_num%(%pt%,pt2,'value') and %link_plot%(%pt%,pt2) and not %link_noplot%(%pt%,pt2)  ),

  if (minval=maxval,
    link_color(%pt%,pt2) = irgb("%color%");
*    link_width(%pt%,pt2)$(%width%=0) = %width%;
     link_width(%pt%,pt2) = %width%;
  else
$   batinclude "%gams.scrdir%get_width.scr"   %link_num%(%pt%,pt2,'value')  %scaleopt%
    link_width(%pt%,pt2) = iwidth;

    linknumber = %link_num%(%pt%,pt2,'value');

    if(%link_num%(%pt%,pt2,'value')>0,
      link_color(%pt%,pt2) = irgb("%link_colp%");
    else
      link_color(%pt%,pt2) = irgb("%link_coln%");
    );
  );

);


* Add header to .MIF/.MID files
Parameter link /1/;
$batinclude "%gams.scrdir%fheader.scr" %fname% link


* Write data to .MIF and .MID files
Parameter visited(%pt%,pt2);
visited(%pt%,pt2) = no; 

loop((%pt%,pt2)$(%link_num%(%pt%,pt2,'value')),

* Flow goes from %pt% --> pt2
  if ((link_width(%pt%,pt2) and not link_width(pt2,%pt%)),
    put fmif /'Line ' (%pt_coord%(%pt%,'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(%pt%,'y')+%link_yoffset%):0:6
                ' ' (%pt_coord%(pt2, 'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(pt2, 'y')+%link_yoffset%):0:6
      / '    Pen (' link_width(%pt%,pt2):0:0;
       if(%linkstyle%,
          put fmif ',' %linkstyle%:0:0;
       else
          put fmif ',' %arrow%:0:0;
       );
       put fmif    ',' link_color(%pt%,pt2) ')';
    if(num_ident,
      put$(%link_num%(%pt%,pt2,'value'))    fmid '"' %pt%.tl:0 '.' pt2.tl:0 '", "' pt2.tl:0 '","' %link_num%(%pt%,pt2,'value') '"' /;
      put$(%link_num%(%pt%,pt2,'value')=0)  fmid '"' %pt%.tl:0 '.' pt2.tl:0 '", "' pt2.tl:0 '", ""' /;
    else
      put fmid '"' %pt%.tl:0 '", "' pt2.tl:0  '"'/;
    );

* Flow goes from pt2 --> %pt%
  elseif (link_width(pt2,%pt%) and not link_width(%pt%,pt2)),
    put fmif /'Line ' (%pt_coord%(pt2,'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(pt2,'y')+%link_yoffset%):0:6
                ' ' (%pt_coord%(%pt%, 'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(%pt%, 'y')+%link_yoffset%):0:6
      / '    Pen (' link_width(pt2,%pt%):0:0;
         if(%linkstyle%,
            put fmif ',' %linkstyle%:0:0;
         else
            put fmif ',' %rev_arrow%:0:0;
         );
         put  ',' link_color(pt2,%pt%) ')';
    if(num_ident,
      put$(%link_num%(pt2,%pt%,'value'))    fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0 '","' %link_num%(pt2,%pt%,'value') '"' /;
      put$(%link_num%(pt2,%pt%,'value')=0)  fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0 '", ""' /;
    else
      put fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0  '"'/;
    );

* Flow goes pt2 <--> %pt%
   elseif (link_width(pt2,%pt%) and link_width(%pt%,pt2) and not visited(%pt%,pt2) ),

*   Status log report for bidrectional flow
    if ( %link_num%(%pt%,pt2,'value') <> 1,
      put$(runi=0) flog /'Bidirectional flow for the following requested links:'/;
      runi = 1;
    );

* We choose the color and width of %pt%, since %pt% > pt2
    if ( abs(%link_num%(%pt%,pt2,'value')) >= abs(%link_num%(pt2,%pt%,'value')),

      if ( %link_num%(%pt%,pt2,'value') <> 1,
        put flog '  Color and width shown are from ' %pt%.tl:0 ' --> ' pt2.tl:0 ' for link ' %pt%.tl:0 ' <--> ' pt2.tl:0 /;
        put flog '     Numerical values are ' %pt%.tl:0 ' --> 'pt2.tl:0 ' : ' %link_num%(%pt%,pt2,'value') /;
        put flog '                          ' pt2.tl:0 ' --> '%pt%.tl:0 ' : ' %link_num%(pt2,%pt%,'value') /;
      );

      put fmif /'Line ' (%pt_coord%(%pt%,'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(%pt%,'y')+%link_yoffset%):0:6
                ' ' (%pt_coord%(pt2, 'x')+%link_xoffset%):0:6 ' ' (%pt_coord%(pt2, 'y')+%link_yoffset%):0:6
        / '    Pen (' link_width(%pt%,pt2):0:0;
           if(%linkstyle%,
               put fmif ',' %linkstyle%:0:0;
           else
               put fmif ',' %dbl_arrow%:0:0;
           );
           put fmif  ',' link_color(%pt%,pt2) ')';
      if(num_ident,
        put$(%link_num%(%pt%,pt2,'value'))    fmid '"' %pt%.tl:0 '.' pt2.tl:0 '", "' pt2.tl:0 '","' %link_num%(%pt%,pt2,'value') '"' /;
        put$(%link_num%(%pt%,pt2,'value')=0)  fmid '"' %pt%.tl:0 '.' pt2.tl:0 '", "' pt2.tl:0 '", ""' /;
      else
        put fmid '"' %pt%.tl:0 '.' pt2.tl:0 '", "' pt2.tl:0  '"'/;
      );
* We choose the color and width of p2, since pt2 > %pt%
    else

      put flog '  Color and width shown are from ' pt2.tl:0 ' --> ' %pt%.tl:0 ' for link ' pt2.tl:0 ' <--> ' %pt%.tl:0 /;
      put flog '     Numerical values are ' pt2.tl:0 ' --> '%pt%.tl:0 ' : ' %link_num%(pt2,%pt%,'value') /;
      put flog '                          '%pt%.tl:0 ' --> ' pt2.tl:0 ' : ' %link_num%(%pt%,pt2,'value') /;



      put fmif /'Line ' (%pt_coord%(pt2,'x')+ (%link_xoffset%)):0:6 ' ' (%pt_coord%(pt2,'y')+ (%link_yoffset%)):0:6
                ' ' (%pt_coord%(%pt%, 'x')+ (%link_xoffset%)):0:6 ' ' (%pt_coord%(%pt%, 'y')+ (%link_yoffset%)):0:6
        / '    Pen (' link_width(pt2,%pt%):0:0;
           if(%linkstyle%,
               put fmif ',' %linkstyle%:0:0;
           else
               put fmif ',' %dbl_arrow%:0:0;
           )
           put fmif  ',' link_color(pt2,%pt%) ')';
      if(num_ident,
        put$(%link_num%(pt2,%pt%,'value'))    fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0 '","' %link_num%(pt2,%pt%,'value') '"' /;
        put$(%link_num%(pt2,%pt%,'value')=0)  fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0 '", ""' /;
      else
        put fmid '"' pt2.tl:0 '.' %pt%.tl:0 '", "' %pt%.tl:0  '"'/;
      );

   );

  );
 
  visited(%pt%,pt2) = yes;
  visited(pt2,%pt%) = yes;

);

* Close the .MID/.MIF files
putclose fmid;
putclose fmif;

Alias(uni2,*);
runi=0;

display pt2;


loop((uni,uni2)$( num_ident and (%link_num%(uni,uni2,'value') or %link_num%(uni2,uni,'value')) and not (%pt%(uni)) ),
*loop((uni,uni2)$( num_ident and (%link_num%(uni,uni2,'value') or %link_num%(uni2,uni,'value')) and not (%pt%(uni) or pt2(uni) ) ),
  put$(runi=0) flog /"The following requested links in the data file have no coordinate data:"
  if (mod(runi,6)=0, put /);
  put flog '  ' uni.tl:0, ' --> ' uni2.tl:0;
  runi = runi + 1;
  has_errors = YES;
)



put$(not has_errors) flog / "Map created succesfully!"
putclose flog;