*===============================================================================
$ontext
FILENAME:      write_point.gms
DATE:          7 May, 2002
PURPOSE:       Creates the MIF/MID files depicting points (plants, centroids of
                         states, centroids of control areas, etc.)
INPUT PARAMETERS:
               pt        -  set of points (either plants, centroids of states,
                                 control areas, counties, etc.)
               pt_coord  -  set of coordinate data for pt.
               pt_num    -  set of points with numerical values in data file
                                 for which mapping data exists
               pt_plot   -  set of points to actually be plotted
               pt_noplot -  set of points to be ignored
               fname     -  .mif and .mid root filename
               size      -  size of point
               color     -  color of point interior
               colp      -  shading color of positive numerical values in pt_num
               coln      -  shading color of negative numerical values in pt_num
               pt_type   -  if a data file with numerical values is specified,
                                 the points can be displayed by size or by shading
               pt_xoffset-  if specified, x-direction offset from actual coordinate data
               pt_yoffset-  if specified, y-direction offset from actual coordinate data
               scaleopt  -  scaling option. 1=[0,max], 2=[min,max]


CALLED BY:
*$batinclude write_plants.gms  pt  pt_coord pt_num  pt_plot  pt_noplot  fname  size...

                         ...color  pt_colp   pt_coln   pt_type  pt_xoffset   pt_yoffset
         
                         ...scaleopt

$offtext

*===============================================================================

$call gmsunzip -Cp GMSMAPDB get_color.gms > "%gams.scrdir%get_color.scr"
$call gmsunzip -Cp GMSMAPDB get_size.gms  > "%gams.scrdir%get_size.scr"


* Input identifiers
$set pt          %1
$set pt_coord    %2
$set pt_num      %3
$set pt_plot     %4
$set pt_noplot   %5
$set fname       %6
$set size        %7
$set ptcolor     %8
$set pt_colp     %9
$set pt_coln    %10
$set pt_type    %11
$set pt_xoffset %12
$set pt_yoffset %13
$set scaleopt   %14



* Identifier if numerical values are to be associated with each pt
num_ident$(card(%pt_num%)>0) = 1;


* Compute max and min ranges for numerical value %pt_num%(pt)

max_p = smax(pt_plot(%pt%)$( (%pt_num%(%pt%)>0) and not pt_noplot(%pt%) ), %pt_num%(%pt%));
max_n = smax(pt_plot(%pt%)$( (%pt_num%(%pt%)<0) and not pt_noplot(%pt%) ), abs(%pt_num%(%pt%)) );

min_p = smin(pt_plot(%pt%)$( (%pt_num%(%pt%)>0) and not pt_noplot(%pt%) ), %pt_num%(%pt%));
min_n = smin(pt_plot(%pt%)$( (%pt_num%(%pt%)<0) and not pt_noplot(%pt%) ), abs(%pt_num%(%pt%)) );

maxval = max(max_p, max_n);
minval = min(min_p, min_n);


pt_color(%pt%) = irgb("%color%");
pt_size(%pt%)$(irgb("%color%")<>irgb("none")) = %size%;


* Define color and size values for regions in data file
if (num_pt_type("%pt_type%")=num_pt_type("color"),
  pt_size(%pt%)$(irgb("%color%")<>irgb("none")) = 0;
  loop(%pt%$( %pt_num%(%pt%) ),
    pt_size(%pt%) = %size%;
    if(%pt_num%(%pt%)>0,
$     batinclude "%gams.scrdir%get_color.scr" %pt_num%(%pt%) %pt_colp%  %scaleopt%
    else
$     batinclude "%gams.scrdir%get_color.scr" %pt_num%(%pt%) %pt_coln%  %scaleopt%
    );
    pt_color(%pt%) = icolor;
  );
elseif (num_pt_type("%pt_type%")=num_pt_type("size")),
  loop(%pt%$( %pt_num%(%pt%) ),
    if(%pt_num%(%pt%)>0,
      if (maxval=minval,
*        use user specified values or default
      else
$       batinclude "%gams.scrdir%get_size.scr" %pt_num%(%pt%)  %scaleopt%
        pt_size(%pt%) = isize;
        pt_color(%pt%) = irgb("%pt_colp%");
      );
    else
$     batinclude "%gams.scrdir%get_size.scr" %pt_num%(%pt%)  %scaleopt%
      pt_size(%pt%) = isize;
      pt_color(%pt%) = irgb("%pt_coln%");
    );
  );
);



* Add header to .MIF/.MID files
Parameter link /0/;
$batinclude "%gams.scrdir%fheader.scr" %fname%  link


* Write data to .MIF and .MID files
display pt_size;

loop(%pt%$(%pt_plot%(%pt%) and not %pt_noplot%(%pt%) and pt_size(%pt%)>0 ),
  put fmif /'Point ' (%pt_coord%(%pt%,'x')+(%pt_xoffset%) ):0:6 ' ' (%pt_coord%(%pt%,'y')+(%pt_yoffset%)):0:6  /
         '    Symbol ('
                       34:0:0 ','
                       pt_color(%pt%):0:0 ','
                       pt_size(%pt%):0:0 ')';
  if(num_ident,
    put$(%pt_num%(%pt%))    fmid '"' %pt%.tl:0 '","' %pt%name.te(%pt%):0 '","' %pt_num%(%pt%) '"' /;
    put$(%pt_num%(%pt%)=0)  fmid '"' %pt%.tl:0 '","' %pt%name.te(%pt%):0 '",""' /;
  else
    put fmid '"' %pt%.tl:0 '"'  ',"' %pt%name.te(%pt%):0 '"'/;
  );
);

* Close the .MID/.MIF files
putclose fmid;
putclose fmif;


* Create status report
put flog ''; // deletes flog
parameter runi /0/;


loop(uni$(num_ident and %pt_num%(uni) and not %pt%(uni)),
  put$(runi=0) /"The following requested points in the data file have no region data:"
  if (mod(runi,12)=0, put /);
  put uni.tl:6:0;
  runi = runi + 1;
  has_errors = YES;
)


put$(not has_errors) / "Map created succesfully!"
putclose flog;
