*===============================================================================
$ontext
FILENAME:      write_region.gms
DATE:          7 May, 2002
PURPOSE:       Creates the MIF/MID files depicting regions(states, counties, etc.)
INPUT PARAMETERS:
               reg       -  set of regions (either states, control areas,
                                 counties, etc.)
               reg_num   -  set of regions with numerical values in data file
                                 for which mapping data exists
               reg_plot  -  set of regions to actually be plotted
               reg_noplot-  set of regions to be ignored
               fname     -  .mif and .mid root filename
               width     -  width of region border
               color     -  color of region interior
               shading   -  numerical value (for Mapinfo Brush pattern), resulting
                                 in either a fill, transparent or no pattern.
               colp      -  shading color of positive numerical values in reg_num
               coln      -  shading color of negative numerical values in reg_num
               scaleopt  -  scaling option. 1=[0,max], 2=[min,max]
               colorfile -  color file for region (if defined)
               reg_color_tmp
                         - if colorfile defined then region color (rgb value)

CALLED BY:
*$batinclude write_region   reg  reg_num  reg_plot  reg_noplot  fname  width  color...

                               ...reg_shading  reg_colp  reg_coln   scaleopt  colorfile

                               ...reg_color_tmp

$offtext

*===============================================================================

$call gmsunzip -Cp GMSMAPDB get_color.gms > "%gams.scrdir%get_color.scr"

* Input identifiers
$set reg            %1
$set reg_num        %2
$set reg_plot       %3
$set reg_noplot     %4
$set fname          %5
$set width          %6
$set reg_color      %7
$set reg_shading    %8
$set reg_colp       %9
$set reg_coln      %10
$set scaleopt      %11
$set colorfile     %12
$set reg_color_tmp %13

set no_colorfile /none/;
parameter colorfile_def(*);
colorfile_def("%colorfile%") = 1;
colorfile_def("none") = 0;


* Identifier if numerical values are to be associated with each region
num_ident$(card(%reg_num%)>0) = 1;


* Compute max and min ranges for numerical value %reg_num%(reg)

max_p = smax( reg_plot(%reg%)$( (%reg_num%(%reg%)>0) and not reg_noplot(%reg%)) , %reg_num%(%reg%) );
max_n = smax( reg_plot(%reg%)$( (%reg_num%(%reg%)<0) and not reg_noplot(%reg%)), abs(%reg_num%(%reg%)) );

min_p = smin( reg_plot(%reg%)$( (%reg_num%(%reg%)>0) and not reg_noplot(%reg%)), %reg_num%(%reg%));
min_n = smin( reg_plot(%reg%)$( (%reg_num%(%reg%)<0) and not reg_noplot(%reg%)), abs(%reg_num%(%reg%)) );

maxval = max(max_p, max_n);
minval = min(min_p, min_n);



* Define values for all regions in set "reg"
Parameter reg_color(%reg%);

if(maxval=minval,
  reg_color(%reg%) = irgb("none");
else
  reg_color(%reg%) = irgb("%color%");
);


* Define values for regions in data file

loop(%reg%$( %reg_num%(%reg%) ),
  if(%reg_num%(%reg%)<>0,
    if(maxval=minval,
      reg_color(%reg%) = irgb("%color%");
    elseif(%reg_num%(%reg%)>0),
$     batinclude "%gams.scrdir%get_color.scr" %reg_num%(%reg%) %reg_colp%  %scaleopt%
      reg_color(%reg%) = icolor;
    else
$     batinclude "%gams.scrdir%get_color.scr" %reg_num%(%reg%) %reg_coln%   %scaleopt%
      reg_color(%reg%) = icolor;
    );
  );
);


* If a --colorfile is defined use these as region color values
if(colorfile_def("%colorfile%"),
  reg_color(%reg%) = %reg_color_tmp%(%reg%);
);


* Add header to .MIF/.MID files
$batinclude "%gams.scrdir%fheader.scr" %fname%  0

file fbat /app.bat/;

* Write data to .MIF and .MID files
loop(%reg%$(%reg_plot%(%reg%) and not %reg_noplot%(%reg%) ),
  putclose fmif;
  putclose fbat 'gmsunzip -Cp GMSMAPDB ' %reg%.te(%reg%) ' >> %fname%.mif';
  execute "app.bat > nul";
*  put_utilities fmif 'inc' / %reg%.te(%reg%);
  put fmif  '    Pen (%width%,2,0)'
          / '    Brush(' %reg_shading%:0:0 ',' reg_color(%reg%):0:0 ')';

  if(num_ident,
    put$%reg_num%(%reg%)     fmid '"' %reg%.tl:0 '","' %reg%name.te(%reg%):0 '", " '%reg_num%(%reg%) '"' /;
    put$(%reg_num%(%reg%)=0) fmid '"' %reg%.tl:0 '","' %reg%name.te(%reg%):0 '", ""' /;
  else
    put fmid '"' %reg%.tl:0 '"'  ',"' %reg%name.te(%reg%):0 '"'/;
  );
);

* Close the .MID/.MIF files
putclose fmid;
putclose fmif;


* Create status report
put flog ''; // deletes flog
parameter runi /0/;


loop(uni$(num_ident and %reg_num%(uni) and not %reg%(uni)),
  put$(runi=0) /"The following requested regions in the data file have no region data:"
  if (mod(runi,12)=0, put /);
  put uni.tl:6:0;
  runi = runi + 1;
  has_errors = YES;
)


put$(not has_errors) / "Map created succesfully!"
putclose flog;


